@@ -1,11 +1,11 @@
 package Minion::Backend::MongoDB;
-$Minion::Backend::MongoDB::VERSION = '1.02';
+$Minion::Backend::MongoDB::VERSION = '1.03';
 # ABSTRACT: MongoDB backend for Minion
 
 use Mojo::Base 'Minion::Backend';
 
 use boolean;
-use BSON::ObjectId;
+use BSON::OID;
 use BSON::Types ':all';
 use DateTime;
 use DateTime::Set;
@@ -30,7 +30,10 @@ sub broadcast {
   my ($s, $command, $args, $ids) = (shift, shift, shift || [], shift || []);
 
   my $match = {};
-  $match->{_id} = {'$in' => $ids} if (scalar(@$ids));
+
+  my @ids = map $s->_oid($_), @$ids;
+
+  $match->{_id} = {'$in' => \@ids} if (scalar(@ids));
 
   my $res = $s->workers->update_many(
     $match, {'$push' => {inbox => [$command, @$args]}}
@@ -40,23 +43,21 @@ sub broadcast {
 }
 
 sub dequeue {
-  my ($self, $oid, $wait, $options) = @_;
+  my ($self, $id, $wait, $options) = @_;
 
-  if ((my $job = $self->_try($oid, $options))) { return $job }
+  if ((my $job = $self->_try($id, $options))) { return $job }
   return undef if Mojo::IOLoop->is_running;
 
   # Capped collection for notifications
   $self->_notifications;
 
   my $timer = Mojo::IOLoop->timer($wait => sub { Mojo::IOLoop->stop });
-  my $recur = Mojo::IOLoop->recurring(1 => sub {
+  Mojo::IOLoop->delay(sub {
       Mojo::IOLoop->stop if ($self->_await)
-  });
-  Mojo::IOLoop->start;
-  Mojo::IOLoop->remove($recur);
+  })->wait;
   Mojo::IOLoop->remove($timer);
 
-  return $self->_try($oid, $options);
+  return $self->_try($id, $options);
 
 }
 
@@ -68,6 +69,9 @@ sub enqueue {
   # Capped collection for notifications
   $self->_notifications;
 
+  @{$options->{parents}} = map $self->_oid($_), @{$options->{parents}}
+    if (exists $options->{parents});
+
   my $doc = {
     args    => $args,
     created => DateTime->from_epoch(epoch => time),
@@ -83,9 +87,9 @@ sub enqueue {
   };
 
   my $res = $self->jobs->insert_one($doc);
-  my $oid = $res->inserted_id;
+  my $id = $res->inserted_id->hex;
   $self->notifications->insert_one({c => 'created'});
-  return $oid;
+  return $id;
 }
 
 sub fail_job { shift->_update(1, @_) }
@@ -151,7 +155,7 @@ sub list_jobs {
   my ($self, $lskip, $llimit, $options) = @_;
 
   my $imatch    = {};
-  $options->{'_ids'} = [map(BSON::ObjectId->new($_), @{$options->{ids}})]
+  $options->{'_ids'} = [map($self->_oid($_), @{$options->{ids}})]
     if $options->{ids};
   foreach (qw(_id state task queue)) {
       $imatch->{$_} = {'$in' => $options->{$_ . 's'}} if $options->{$_ . 's'};
@@ -236,7 +240,14 @@ sub list_locks {
 sub list_workers {
   my ($self, $offset, $limit, $options) = @_;
 
-  my $cursor = $self->workers->find({pid => {'$exists' => true}});
+  my $match    = {};
+  $options->{'_ids'} = [map($self->_oid($_), @{$options->{ids}})]
+    if $options->{ids};
+  foreach (qw(_id)) {
+      $match->{$_} = {'$in' => $options->{$_ . 's'}} if $options->{$_ . 's'};
+  }
+
+  my $cursor = $self->workers->find($match);
   my $total = scalar($cursor->all);
   $cursor->reset;
   $cursor->sort({_id => -1})->skip($offset)->limit($limit);
@@ -251,7 +262,7 @@ sub lock {
 
 sub new {
   my ($class, $url) = (shift, shift);
-  my $client = MongoDB::MongoClient->new(host => $url, @_);
+  my $client = MongoDB->connect($url, @_);
   my $db = $client->db($client->db_name);
 
   my $self = $class->SUPER::new(dbclient => $client, mongodb => $db);
@@ -264,16 +275,17 @@ sub new {
 
 sub note {
   my ($self, $id, $merge) = @_;
-
-  return 1 unless defined $merge;
+  return 0 if (!keys %$merge);
   my $set = {};
   my $unset = {};
   while (my ($k, $v) = each %$merge) {
       (defined $v ? $set : $unset)->{"notes.$k"} = $v;
   };
-  my @update = ( {_id => $id} );
-  push @update, {'$set' => $set} if (keys %$set);
-  push @update, {'$unset' => $unset} if (keys %$unset);
+  my @update = ( {_id => $self->_oid($id)} );
+  my %set_unset;
+  $set_unset{'$set'}    = $set if (keys %$set);
+  $set_unset{'$unset'}  = $unset if (keys %$unset);
+  push @update, \%set_unset;
   push @update, {
       upsert    => 0,
       returnDocument => 'after',
@@ -284,7 +296,7 @@ sub note {
 sub receive {
   my ($self, $id) = @_;
   my $oldrec = $self->workers->find_one_and_update(
-    {_id => $id, inbox => { '$exists' => 1, '$ne' => [] } },
+    {_id => $self->_oid($id), inbox => { '$exists' => 1, '$ne' => [] } },
     {'$set' => {inbox => [] }},
     {
         upsert    => 0,
@@ -301,13 +313,15 @@ sub register_worker {
   return $id
     if $id
     && $self->workers->find_one_and_update(
-    {_id => $id}, {'$set' => {
+    {_id => $self->_oid($id)}, {'$set' => {
         notified => DateTime->from_epoch(epoch => time),
         status => $options->{status} // {}
     }});
 
+  # indexes for jobs
   $self->jobs->indexes->create_one(Tie::IxHash->new(state => 1, delayed => 1, task => 1, queue => 1));
   $self->jobs->indexes->create_one(Tie::IxHash->new(finished => 1));
+  # indexes for locks
   $self->locks->indexes->create_one(Tie::IxHash->new(name => 1), {unique => 1});
   $self->locks->indexes->create_one(Tie::IxHash->new(expires => 1));
   my $res = $self->workers->insert_one(
@@ -319,13 +333,12 @@ sub register_worker {
       inbox => [],
     }
   );
-
-  return $res->inserted_id;
+  return $res->inserted_id->hex;
 }
 
 sub remove_job {
-  my ($self, $oid) = @_;
-  my $doc = {_id => $oid, state => {'$in' => [qw(failed finished inactive)]}};
+  my ($self, $id) = @_;
+  my $doc = {_id => $self->_oid($id), state => {'$in' => [qw(failed finished inactive)]}};
   return !!$self->jobs->delete_one($doc)->deleted_count;
 }
 
@@ -368,12 +381,12 @@ sub repair {
 sub reset { $_->drop for $_[0]->workers, $_[0]->jobs, $_[0]->locks }
 
 sub retry_job {
-  my ($self, $oid, $retries, $options) = (shift, shift, shift, shift || {});
+  my ($self, $id, $retries, $options) = (shift, shift, shift, shift || {});
   $options->{delay} //= 0;
 
   my $dt_now = DateTime->now();
 
-  my $query = {_id => $oid, retries => $retries};
+  my $query = {_id => $self->_oid($id), retries => $retries};
   my $update = {
     '$inc' => {retries => 1},
     '$set' => {
@@ -432,7 +445,7 @@ sub unlock {
 
     return defined $doc ;
 }
-sub unregister_worker { shift->workers->delete_one({_id => shift}) }
+sub unregister_worker { $_[0]->workers->delete_one({_id => $_[0]->_oid($_[1])}) }
 
 sub worker_info { $_[0]->_worker_info($_[0]->workers->find_one({_id => $_[1]})) }
 
@@ -446,9 +459,9 @@ sub _await {
         { c => 'created'},
         { c => 'update_retries'}
     ]
-  })->tailable(1);
-  return undef unless my $doc = $cursor->next || $cursor->next;
-  $self->{last} = $doc->{_id};
+  })->tailable_await(1);
+  return undef unless my @doc = $cursor->all;
+  $self->{last} = $doc[-1]->{_id};
   return 1;
 }
 
@@ -457,7 +470,7 @@ sub _job_info {
 
   return undef unless my $job = shift;
 
-  $job->{id}        = $job->{_id};
+  $job->{id}        = $job->{_id}->hex;
   $job->{retries} //= 0;
   $job->{children}  = [map $_->{_id}->hex, @{$job->{children}}];
   $job->{time}      = DateTime->now->epoch; # server time
@@ -531,13 +544,17 @@ sub _notifications {
   $notifications->insert_one({});
 }
 
+sub _oid {
+	return defined $_[1] ? BSON::OID->new(oid => pack("H*", $_[1])) : undef;
+}
+
 sub _total {
     my ($name, $res, $tot) = @_;
     return { total => $tot, $name => $res};
 }
 
 sub _try {
-  my ($self, $oid, $options) = @_;
+  my ($self, $id, $options) = @_;
 
   my $match = Tie::IxHash->new(
     delayed   => {'$lt' => DateTime->from_epoch(epoch => time)},
@@ -545,7 +562,7 @@ sub _try {
     task      => {'$in' => [keys %{$self->minion->tasks}]},
     queue     => {'$in' => $options->{queues} // ['default']}
   );
-  $match->Push('_id' => $options->{id}) if defined $options->{id};
+  $match->Push('_id' => $self->_oid($options->{id})) if defined $options->{id};
 
   my $docs = $self->jobs->find($match)->sort({priority => -1, id => 1});
 
@@ -569,7 +586,7 @@ sub _try {
     {'$set' => {
         started => DateTime->from_epoch(epoch => time),
         state => 'active',
-        worker => $oid
+        worker => $self->_oid($id)
     }},
     {
         projection      => {args => 1, retries => 1, task => 1},
@@ -580,19 +597,19 @@ sub _try {
 
   my $job = $self->jobs->find_one_and_update(@$doc);
   return undef unless ($job->{_id});
-  $job->{id} = $job->{_id};
+  $job->{id} = $job->{_id}->hex;
   return $job;
 }
 
 sub _update {
-  my ($self, $fail, $oid, $retries, $result) = @_;
+  my ($self, $fail, $id, $retries, $result) = @_;
 
   my $update = {
       finished => DateTime->now,
       state => $fail ? 'failed' : 'finished',
       result => $fail ?  $result . '' : $result ,
   };
-  my $query = {_id => $oid, state => 'active', retries => $retries};
+  my $query = {_id => $self->_oid($id), state => 'active', retries => $retries};
   my $doc = $self->jobs->find_one_and_update($query, {'$set' => $update},
     {returnDocument => 'after'});
 
@@ -601,7 +618,7 @@ sub _update {
   return 1 if !$fail || (my $attempts = $doc->{attempts}) == 1;
   return 1 if $retries >= ($attempts - 1);
   my $delay = $self->minion->backoff->($retries);
-  return $self->retry_job($oid, $retries, {delay => $delay});
+  return $self->retry_job($id, $retries, {delay => $delay});
 }
 
 sub _worker_info {
@@ -614,8 +631,8 @@ sub _worker_info {
 
   return {
     host     => $worker->{host},
-    id       => $worker->{_id},
-    jobs     => [map { $_->{_id} } $cursor->all],
+    id       => $worker->{_id}->hex,
+    jobs     => [map { $_->{_id}->hex } $cursor->all],
     pid      => $worker->{pid},
     started  => $worker->{started}->epoch,
     notified => $worker->{notified}->epoch,
@@ -638,7 +655,7 @@ Minion::Backend::MongoDB - MongoDB backend for Minion
 
 =head1 VERSION
 
-version 1.02
+version 1.03
 
 =head1 SYNOPSIS
 
@@ -650,7 +667,7 @@ version 1.02
 
 L<Minion::Backend::MongoDB> is a L<MongoDB> backend for L<Minion>
 derived from L<Minion::Backend::Pg> and supports its methods and tests
-up to 9.11.
+up to 9.13 (2019-08-29).
 
 =head1 ATTRIBUTES
 
@@ -1141,6 +1158,13 @@ Unregister worker.
 
 Get information about a worker or return C<undef> if worker does not exist.
 
+=head2 _oid
+
+  my $mongo_oid = $backend->_oid($hex_24length);
+
+EXPERIMENTAL: Convert an 24-byte hexadecimal value into a C<BSON::OID> object.
+Usually, it should be used only if you need to query the MongoDB directly
+
 =head1 NOTES ABOUT USER
 
 User must have this roles
